package at.ac.uibk.gitsearch.domain;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * A Review.
 */
@Entity
@Table(name = "reviews")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
@SuppressWarnings({ "PMD.AvoidFieldNameMatchingMethodName", "PMD.AvoidDuplicateLiterals", "PMD.ShortMethodName", "PMD.TooManyMethods" })
public class Review implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    private Long id;

    @Size(max = 255)
    @Column(name = "link", length = 255)
    private String link;

    @Column(name = "requested_by")
    private Long requestedBy;

    @NotNull
    @Size(max = 150)
    @Column(name = "resource", length = 150, nullable = false, unique = true)
    private String resource;

    @Column(name = "resourceid")
    private String resourceid;

    @OneToMany(mappedBy = "review", fetch = FetchType.EAGER)
    @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
    @JsonIgnoreProperties(value = { "review", "user", "histories" }, allowSetters = true)
    private Set<ReviewRating> comments = new HashSet<>();

    @OneToMany(mappedBy = "review", cascade = CascadeType.ALL)
    @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
    @JsonIgnoreProperties(value = { "review", "reviewRating" }, allowSetters = true)
    private Set<ReviewHistoryEntry> histories = new HashSet<>();

    @OneToMany(mappedBy = "review", cascade = CascadeType.REMOVE)
    private Set<ChatMessage> chatMessages = new HashSet<>();

    // jhipster-needle-entity-add-field - JHipster will add fields here

    public Long getId() {
        return this.id;
    }

    public Set<ChatMessage> getChatMessages() {
        return chatMessages;
    }

    public void setChatMessages(Set<ChatMessage> chatMessages) {
        this.chatMessages = chatMessages;
    }

    public Review id(Long id) {
        this.setId(id);
        return this;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getLink() {
        return this.link;
    }

    public Review link(String link) {
        this.setLink(link);
        return this;
    }

    public void setLink(String link) {
        this.link = link;
    }

    public Long getRequestedBy() {
        return this.requestedBy;
    }

    public Review requestedBy(Long requestedBy) {
        this.setRequestedBy(requestedBy);
        return this;
    }

    public void setRequestedBy(Long requestedBy) {
        this.requestedBy = requestedBy;
    }

    public String getResource() {
        return this.resource;
    }

    public Review resource(String resource) {
        this.setResource(resource);
        return this;
    }

    public void setResource(String resource) {
        this.resource = resource;
    }

    public String getResourceID() {
        return this.resourceid;
    }

    public Review resourceID(String resourceid) {
        this.setResourceID(resourceid);
        return this;
    }

    public void setResourceID(String resourceid) {
        this.resourceid = resourceid;
    }

    public Set<ReviewRating> getComments() {
        return this.comments;
    }

    public void setComments(Set<ReviewRating> reviewRatings) {
        if (this.comments != null) {
            this.comments.forEach(i -> i.setReview(null));
        }
        if (reviewRatings != null) {
            reviewRatings.forEach(i -> i.setReview(this));
        }
        this.comments = reviewRatings;
    }

    public Review comments(Set<ReviewRating> reviewRatings) {
        this.setComments(reviewRatings);
        return this;
    }

    public Review addComments(ReviewRating reviewRating) {
        this.comments.add(reviewRating);
        reviewRating.setReview(this);
        return this;
    }

    public Review removeComments(ReviewRating reviewRating) {
        this.comments.remove(reviewRating);
        reviewRating.setReview(null);
        return this;
    }

    public Set<ReviewHistoryEntry> getHistories() {
        return this.histories;
    }

    public void setHistories(Set<ReviewHistoryEntry> reviewHistoryEntries) {
        if (this.histories != null) {
            this.histories.forEach(i -> i.setReview(null));
        }
        if (reviewHistoryEntries != null) {
            reviewHistoryEntries.forEach(i -> i.setReview(this));
        }
        this.histories = reviewHistoryEntries;
    }

    public Review histories(Set<ReviewHistoryEntry> reviewHistoryEntries) {
        this.setHistories(reviewHistoryEntries);
        return this;
    }

    public Review addHistory(ReviewHistoryEntry reviewHistoryEntry) {
        this.histories.add(reviewHistoryEntry);
        reviewHistoryEntry.setReview(this);
        return this;
    }

    public Review removeHistory(ReviewHistoryEntry reviewHistoryEntry) {
        this.histories.remove(reviewHistoryEntry);
        reviewHistoryEntry.setReview(null);
        return this;
    }

    // jhipster-needle-entity-add-getters-setters - JHipster will add getters and
    // setters here

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Review)) {
            return false;
        }
        return id != null && id.equals(((Review) o).id);
    }

    @Override
    public int hashCode() {
        // see
        // https://vladmihalcea.com/how-to-implement-equals-and-hashcode-using-the-jpa-entity-identifier/
        return getClass().hashCode();
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "Review{" +
                "id=" + getId() +
                ", link='" + getLink() + "'" +
                ", requestedBy=" + getRequestedBy() +
                ", resource='" + getResource() + "'" +
                ", resourceid='" + getResourceID() + "'" +
                "}";
    }
}
