/* eslint-disable */
import { Injectable } from '@angular/core';
import { HttpClient, HttpParams, HttpResponse } from '@angular/common/http';
import { Observable } from 'rxjs';

import { ApplicationConfigService } from 'app/core/config/application-config.service';
import { SearchResultsDTO } from 'app/shared/model/search/search-results-dto.model';
import { SearchInput } from 'app/shared/model/search/search-input.model';
import { Statistics } from 'app/shared/model/statistics.model';
import { encodeURIforExerciseId } from 'app/exercise/service/exercise.service';
import { ReviewRequest } from 'app/admin/review-management/reviewRequest.model';
import { SearchResultDTO } from 'app/shared/model/search/search-result-dto.model';

@Injectable({ providedIn: 'root' })
export class SearchService {
  public resourceSearchUrlPageDetails = this.applicationConfigService.getEndpointFor('api/search/page-details');
  public resourseSearchUrlAuthor = this.applicationConfigService.getEndpointFor('api/search/author');
  public resourceSearchUrlStatisticsForExercise = this.applicationConfigService.getEndpointFor('api/statistics/exercise/');
  public resourceKeywordsAutoCompleteDetails = this.applicationConfigService.getEndpointFor('api/search/keywordsAutoComplete');
  public resourceFormatsAutoCompleteDetails = this.applicationConfigService.getEndpointFor('api/search/formatsAutoComplete');
  public resourceProgrammingLanguageAutoCompleteDetails = this.applicationConfigService.getEndpointFor(
    'api/search/programmingLanguageAutoComplete'
  );
  public resourceContributorAutoCompleteDetails = this.applicationConfigService.getEndpointFor('api/search/contributorAutoComplete');
  public resourceContributorCreatorAutoCompleteDetails = this.applicationConfigService.getEndpointFor(
    'api/search/contributorCreatorAutoComplete'
  );
  public resourceCreatorAutoCompleteDetails = this.applicationConfigService.getEndpointFor('api/search/creatorAutoComplete');

  constructor(protected http: HttpClient, private applicationConfigService: ApplicationConfigService) {}

  searchPageDetails(searchInput: SearchInput): Observable<SearchResultsDTO> {
    return this.http.post<SearchResultsDTO>(this.resourceSearchUrlPageDetails, searchInput);
  }

  getStatisticsForExercise(exerciseID: string): Observable<Statistics> {
    return this.http.get<Statistics>(this.resourceSearchUrlStatisticsForExercise + encodeURIforExerciseId(exerciseID));
  }

  getStatisticsForUser(): Observable<Statistics> {
    return this.http.get<Statistics>('api/user/achievements');
  }

  getResultsForAuthor(email: string): Observable<SearchResultsDTO> {
    return this.http.post<SearchResultsDTO>(this.resourseSearchUrlAuthor, email);
  }

  getResultsForLoggedInUser(): Observable<SearchResultsDTO> {
    return this.http.get<SearchResultsDTO>(this.resourseSearchUrlAuthor);
  }

  getAllResources(): Observable<SearchResultDTO[]> {
    return this.http.get<SearchResultDTO[]>('api/search/all');
  }

  requestReview(exercise: ReviewRequest): Observable<ReviewRequest> {
    return this.http.post<ReviewRequest>(this.applicationConfigService.getEndpointFor('/api/requestReview/'), exercise);
  }

  /**
   * Used to export an exercise as a compressed zip file
   * The file will contain all files as well as the exercise text and further metadata, except for protected files/forlder
   * @param exerciseId
   */
  exportProject(exerciseId: string, recursion: string): Observable<HttpResponse<Blob>> {
    const parameters = new HttpParams().set('recursion', recursion);
    return this.http.post(this.applicationConfigService.getEndpointFor('/api/exerciseFiles/') + encodeURIforExerciseId(exerciseId), '', {
      params: parameters,
      observe: 'response',
      responseType: 'blob',
    });
  }

  /**
 unused!!
  downloadFile(projectID: string): Observable<Object> {
    return this.http.post(this.applicationConfigService.getEndpointFor('download/') + projectID, {
      observe: 'response',
      responseType: 'blob',
    });
  }
*/

  getKeywordsAutoComplete(prefix: string, maxCount: number): Observable<Array<AutoCompletionEntry>> {
    const parameters = new HttpParams().set('keyWordPrefix', prefix).set('max', String(maxCount));
    return this.http.get<Array<AutoCompletionEntry>>(this.resourceKeywordsAutoCompleteDetails, {
      params: parameters,
    });
  }

  getFormatsAutoComplete(prefix: string): Observable<Array<AutoCompletionEntry>> {
    const options: HttpParams = new HttpParams();
    options.append('keywordPrefix', prefix);
    return this.http.get<Array<AutoCompletionEntry>>(this.resourceFormatsAutoCompleteDetails, {
      params: new HttpParams().set('formatPrefix', prefix),
    });
  }

  getProgrammingLanguageAutoComplete(prefix: string): Observable<Array<AutoCompletionEntry>> {
    const options: HttpParams = new HttpParams().set('programmingLanguagePrefix', prefix);
    return this.http.get<Array<AutoCompletionEntry>>(this.resourceProgrammingLanguageAutoCompleteDetails, {
      params: options,
    });
  }

  getContributorAutoComplete(prefix: string): Observable<Array<AutoCompletionEntry>> {
    const options: HttpParams = new HttpParams();
    options.append('keyWordPrefix', prefix);
    return this.http.get<Array<AutoCompletionEntry>>(this.resourceContributorAutoCompleteDetails, {
      params: new HttpParams().set('contributorPrefix', prefix),
    });
  }

  getContributorCreatorAutoComplete(prefix: string, maxCount: number): Observable<Array<AutoCompletionEntry>> {
    const options: HttpParams = new HttpParams().set('contributorPrefix', prefix).set('max', String(maxCount));
    return this.http.get<Array<AutoCompletionEntry>>(this.resourceContributorCreatorAutoCompleteDetails, {
      params: options,
    });
  }

  getCreatorAutoComplete(prefix: string): Observable<Array<AutoCompletionEntry>> {
    const options: HttpParams = new HttpParams().set('contributorPrefix', prefix);
    return this.http.get<Array<AutoCompletionEntry>>(this.resourceCreatorAutoCompleteDetails, {
      params: options,
    });
  }
}

export class AutoCompletionEntry {
  public target: String;
  public hitCount: Number;

  constructor() {
    this.target = '';
    this.hitCount = 0;
  }
}
